
fpoff segment
assume cs:fpoff, ds:fpoff, es:fpoff, ss:fpoff
org 100h

.486

start:  mov  dx,offset message             ; Display message.
        mov  ah,9
        int  21h

        mov  bx,40h                        ; Segment number of BIOS data.
        mov  ds,bx                         ; Move to data segment register.
        and  byte ptr ds:[0010h],11111101b ; Clear the 8087 present bit.

        mov  eax,cr0                       ; Get Control Register 0
        or   al,00000100b                  ; Set the EM bit.
        mov  cr0,eax                       ; Save it back to CPU.

        push cs
        pop  ds                            ; Switch back to CS=DS=SS

        ; CPUID is a new Pentium Instruction.
        ; See Page 25-62 of the Pentium User Manual
        ; Volume 3

        pushfd                             ; Push original EFLAGS.
        pop  eax                           ; Save EFLAGS to EAX.
        mov  ecx,eax                       ; Save EFLAGS to ECX.
        xor  eax,200000h                   ; Flip ID bit in EFLAGS.
        push eax                           ; save for EFLAGS
        popfd                              ; copy to EFLAGS
        pushfd                             ; push EFLAGS
        pop  eax                           ; get new EFLAGS value
        xor  eax, ecx
        jz   exit

        mov  eax,0                         ; Set up for CPUID function 0.
        db 0fh,0a2h                        ; Perform CPUID Function.

        and  ah,0fh                        ; Make sure upper bits zapped.
        mov  _family,ah
        push ax                            ; Save AX
        and  al,0fh                        ; Make sure upper bits zapped.
        mov  _step,al                      ; Save only the step code.
        pop  ax                            ; Restore AX
        shr  al,4                          ; Shift model to lower AL.
        mov  _model,al                     ; Save only model code.

        add  _model,'0'
        add  _step,'0'
        add  _family,'0'

        mov  dx,offset family              ; Point to message.
        mov  ah,9
        int  21h                           ; Call DOS to print to screen.

        mov  dx,offset model               ; Point to message.
        mov  ah,9
        int  21h                           ; Call DOS to print to screen.

        mov  dx,offset step                ; Point to message.
        mov  ah,9
        int  21h                           ; Call DOS to print to screen.

exit:   mov  ah,4ch
        int  21h


message: db "Pentium FPU-OFF and Step info",0dh,0ah
         db "By Alfred J. Heyman, Spectrum Research",0dh,0ah
         db "Compuserve 74471,2476",0dh,0ah
         db "Put into the public domain on 12/21/94",0dh,0ah
         db 0dh,0ah
         db "80x87 Instruction Execution is now off.",0dh,0ah,"$"

step:    db "Processor Stepping: "
_family  db 0
         db 0dh,0ah,"$"

family:  db "Family: "
_step    db 0
         db 0dh,0ah,"$"

model:   db "Model: "
_model   db 0
         db 0dh,0ah,"$"

fpoff ends
end start


